# 敵（Enemy）クラスのプログラム
import random

import pygame as pg


class Enemy:  # 敵作成用のクラス
    # 初期化メソッド
    def __init__(self, screen):
        x = random.randint(100, screen.get_width() - 100)
        y = random.randint(100, screen.get_height() // 4)
        # プロパティ
        self._image = pg.image.load("images/enemy1.png")
        # 画像ファイルが描画されるRectオブジェクトの位置・サイズを指定
        # 初期配置を画面の上の方にランダムで指定（x, y）
        self._rect = pg.Rect(x, y, 50, 50)  # 表示位置
        # randomモジュールのuniform(最小値,最大値)関数は、任意の範囲の少数型の乱数を生成する。
        # 移動量を整数で指定すると、種類が少ないためスピードや進む角度の変化が出にくい。ランダムな少数で指定することにより、微妙な変化を付けられる。
        # 横方向の移動は、左右に同じ様にバラけさせたいため、同じ値のマイナスからプラスの範囲で指定。
        self._vx = random.uniform(-4, 4)  # X軸方向の移動量（速度）-４～４
        # 縦方向の移動は、一度上方向に上がって行って画面上端で反射して落下していく様にするためにマイナスの値からマイナスの値の範囲で指定。
        self._vy = random.uniform(-1, -2)  # Y軸方向の移動量(速度)-１～-３
        self._maxhp = 100  # 最大HP
        self._hp = 100  # 現在のHP

    @property  # _rectプロパティに対するゲッターメソッド
    def rect(self):
        return self._rect

    @rect.setter  # _rectプロパティに対するセッターメソッド
    def rect(self, value):
        self._rect = value

    @property  # _vxプロパティに対するゲッターメソッド
    def vx(self):
        return self._vx

    @vx.setter  # _vxプロパティに対するセッターメソッド
    def vx(self, value):
        self._vx = value

    @property  # _vyプロパティに対するゲッターメソッド
    def vy(self):
        return self._vy

    @vy.setter  # _vyプロパティに対するセッターメソッド
    def vy(self, value):
        self._vy = value

    @property  # _maxhpプロパティに対するゲッターメソッド
    def maxhp(self):
        return self._maxhp

    @property  # _hpプロパティに対するゲッターメソッド
    def hp(self):
        return self._hp

    @hp.setter  # _hpプロパティに対するセッターメソッド
    def hp(self, value):
        self._hp = value

    # メソッド
    def update(self, screen):  # 入力チェックや判断処理
        # 左右の画面端に衝突したかどうかを判定
        if self._rect.x < 0 or self._rect.x > screen.get_width() - 50:
            self._vx = -self._vx
        # 画面上端に衝突したかどうかを判定
        elif self._rect.y < 0:
            self._vy = -self._vy
        self._rect.x += self._vx
        self._rect.y += self._vy

    def draw(self, screen):  # 描画処理
        screen.blit(self._image, self._rect)
        # HPバー
        # 最大HPバーが描画されるRectオブジェクトの位置・サイズを指定
        maxhp_rect = pg.Rect(self._rect.x, self._rect.y - 20, 4, 20)
        hp_height = (self._hp / self._maxhp) * 20  # 現在のHPバーの高さ
        hp_rect = pg.Rect(
            self._rect.x, self._rect.y - hp_height, 4, hp_height
        )  # 現在のHPバーが描画されるRectオブジェクトの位置・サイズを指定
        pg.draw.rect(screen, pg.Color("RED"), maxhp_rect)  # 最大HPバー
        pg.draw.rect(screen, pg.Color("GREEN"), hp_rect)  # 現在のHPバー


# 炎の敵（X軸方向の移動量が少なく、Y軸方向の移動量が多い）
class FlameEnemy(Enemy):
    def __init__(self, screen):
        super().__init__(screen)
        # 上書きするプロパティ
        self._image = pg.image.load("images/enemy2.png")
        self._vx = random.uniform(-2, 2)
        self._vy = random.uniform(-5, -3)


class IceEnemy(Enemy):  # 氷の敵（HPが多い）
    def __init__(self, screen):
        super().__init__(screen)
        # 上書きするプロパティ
        self._image = pg.image.load("images/enemy3.png")
        self._maxhp = 150
        self._hp = 150


class BombEffect:  # 爆発エフェクト
    # 初期化メソッド
    def __init__(self, rect, effects):
        # プロパティ
        self._images = [
            pg.image.load("images/bomb_0.png"),
            pg.image.load("images/bomb_1.png"),
            pg.image.load("images/bomb_2.png"),
            pg.image.load("images/bomb_3.png"),
            pg.image.load("images/bomb_4.png"),
            pg.image.load("images/bomb_5.png"),
        ]  # 爆発エフェクトのアニメーション画像を読み込む
        self._image = self._images[0]
        # GameManagerクラスで作成した爆発エフェクトリスト
        self._effects = effects
        # 敵（Enemyインスタンス）の配置位置（Rectオブジェクト）
        self._rect = rect
        self._cnt = 0  # カウント変数

    # メソッド
    def update(self):  # 更新処理
        self._cnt += 1
        idx = self._cnt // 5
        if idx <= 5:
            self._image = self._images[idx]
        else:  # 爆発エフェクト画像を全部表示し終えたら（self._cnt >= 30）
            self._effects.remove(self)  # 爆発エフェクトリストから自身を削除

    def draw(self, screen):  # 描画処理
        screen.blit(self._image, self._rect)
