# オブザーバーパターン
class Subject:  # 配信者（subject）の基本形
    def __init__(self):
        self._observers = []

    def attach(self, observer):  # 受信者を登録する
        self._observers.append(observer)

    def notify(self, data):  # 受信者に通知する
        for observer in self._observers:
            observer.update(data)


class Publisher(Subject):  # 配信するクラス
    def publish(self, data, text):
        print(f"配信者：{text}")
        self.notify(data)


class Observer:  # 受信者（observer）の基本形
    def update(self, data):
        pass


class Follower(Observer):  # 受信するクラス
    def __init__(self, name):
        self._name = name

    def update(self, data):
        print(f"{self._name}：通知が来た。{data}を見よう！")


p1 = Publisher()  # 送信者（配信者）のインスタンスを作成
f1 = Follower("受信者１")  # 受信者のインスタンスを作成
p1.attach(f1)  # 作成した受信者を送信者（配信者）に登録
f2 = Follower("受信者２")
p1.attach(f2)
while True:
    i = input(
        "配信する動画名を入力して下さい。終了したい時は、「終了」と入力して下さい。"
    )
    if i == "終了":
        break
    else:
        p1.publish(f"{i}", "配信したよ〜。")  # メッセージ送信
