# シングルトンパターン
class HotelFront:  # ホテルのフロント係クラス
    # １つの同じインスタンスしか生成しないことを保証するために、同じ型のインスタンスをprivateなクラス変数として定義する。最初は何も入っていない状態にするためにNoneにしておく。
    _instance = None  # クラス変数

    # 同じ型のインスタンスを返すget_instance（）メソッド（クラスメソッド）を定義する。@classmethodデコレータをつけてメソッドを定義すると、そのメソッドはクラスメソッドになる。クラスメソッドの第１引数は、慣習として「cls」となり、金型である自身のクラスを表す。「cls」を介して、各インスタンス間で共通の金型であるクラスのクラス変数やクラスメソッドにアクセスできる。（対して、通常のメソッドの第１引数である「self」は、クラスから作成された実体であるインスタンスを表す）
    @classmethod
    def get_instance(cls):
        # インスタンス未生成の場合（_instanceの中身がない場合）
        if cls._instance is None:
            cls._instance = cls()  # クラス名()でインスタンスを作成
        return cls._instance

    def checkin(self):
        print("「いらっしゃいませ。デザインパターンホテルへようこそ。」")

    def roomservice(self):
        print("「はい、フロントです。ルームサービスのご注文でしょうか？」")

    def forgotten(self):
        print(
            "「ご連絡ありがとうございます。お名前とお部屋番号をお教えいただけますか？」"
        )


print("フロントでチェックイン")
HotelFront.get_instance().checkin()
print("302号室から内線")
HotelFront.get_instance().roomservice()
print("808号室から内線")
HotelFront.get_instance().roomservice()
print("忘れ物をしたので、外から電話")
HotelFront.get_instance().forgotten()
