# 画面上部のステータス情報に関数プログラム

import pygame as pg


# オブザーバーパターン
class Observer:  # 受信者（observer）の基本形
    # メソッド
    def update(self, ntype):
        pass


class Status(Observer):  # 受信するクラス
    # 初期化メソッド
    def __init__(self):
        # プロパティ
        self.reset()
        self._board = pg.Surface((2000, 36), pg.SRCALPHA)

    # reset()メソッドで初期化されている_scoreプロパティのゲッターメソッド
    @property
    def score(self):
        return self._score

    # メソッド
    def reset(self):  # 表示データのリセット
        self._font = pg.font.Font("fonts/ipaexg.ttf", 32)
        self._distance = 0  # 進んだ距離
        self._score = 0  # スコア（倒した敵の数）

    # 引数「ntype」は、どのタイプの通知なのかを表す
    def update(self, ntype):  # 更新処理
        if ntype == "distance":
            self._distance += 1
        elif ntype == "score":
            self._score += 1

    def draw(self, screen):  # 描画処理
        centerx = screen.get_rect().centerx  # ウィンドウ中央のX座標
        pg.draw.rect(self._board, (0, 0, 0, 128), pg.Rect(0, 0, screen.get_width(), 36))
        screen.blit(self._board, (0, 0))
        info1 = self._font.render(f"経過：{self._distance}", True, pg.Color("WHITE"))
        info2 = self._font.render(f"スコア：{self._score}", True, pg.Color("WHITE"))
        screen.blit(info1, (centerx - 260, 5))
        screen.blit(info2, (centerx + 100, 5))
